/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2010, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: intensity_gradient.h 32774 2010-09-20 12:08:49Z lorenz $
 *
 */
#ifndef PCL_INTENSITY_GRADIENT_H_
#define PCL_INTENSITY_GRADIENT_H_

#include <pcl/features/feature.h>
#include <Eigen3/SVD>

namespace pcl
{
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /** \brief  
   *  \author Michael Dixon
   */
  template <typename PointInT, typename PointNT, typename PointOutT>
  class IntensityGradientEstimation : virtual public FeatureFromNormals<PointInT, PointNT, PointOutT>
  {
    public:
      using Feature<PointInT, PointOutT>::feature_name_;
      using Feature<PointInT, PointOutT>::getClassName;
      using Feature<PointInT, PointOutT>::indices_;
      using Feature<PointInT, PointOutT>::surface_;
      using Feature<PointInT, PointOutT>::k_;
      using Feature<PointInT, PointOutT>::search_parameter_;
      using FeatureFromNormals<PointInT, PointNT, PointOutT>::normals_;

      typedef typename Feature<PointInT, PointOutT>::PointCloudOut PointCloudOut;

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Empty constructor. */
      IntensityGradientEstimation ()
      {
        feature_name_ = "IntensityGradientEstimation";
      };

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Empty destructor. */
      virtual ~IntensityGradientEstimation () {};

    protected:
      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Estimate the intensity gradients for a set of points given in <setInputCloud (), setIndices ()> using 
       *  the surface in setSearchSurface () and the spatial locator in setSearchMethod ().
       *  \param output the resultant point cloud that contains the intensity gradient vectors
       */
      void computeFeature (PointCloudOut &output);

    private:
      void computePointIntensityGradient (const PointCloud<PointInT> &cloud, const std::vector<int> &indices, 
                                          const Eigen3::Vector3f &point, const Eigen3::Vector3f &normal, Eigen3::Vector3f &gradient);
                                      
  };
}

#include "pcl/features/intensity_gradient.hpp"

#endif // #ifndef PCL_INTENSITY_GRADIENT_H_
