/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2009, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: filter.cpp 31038 2010-07-20 19:59:18Z rusu $
 *
 */

/**

\author Radu Bogdan Rusu

**/

#include <sensor_msgs/PointCloud2.h>
#include "pcl/point_types.h"
#include "pcl/filters/filter.h"
#include "pcl/filters/passthrough.h"
#include "pcl/filters/voxel_grid.h"
//#include <pcl/filters/pixel_grid.h>
#include "pcl/filters/extract_indices.h"
#include "pcl/filters/project_inliers.h"
#include "pcl/filters/statistical_outlier_removal.h"
#include "pcl/filters/radius_outlier_removal.h"
//#include <pcl/filters/filter_dimension.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/** \brief Base method for feature estimation for all points given in <setInputCloud (), setIndices ()> using
  * the surface in setSearchSurface () and the spatial locator in setSearchMethod ()
  * \param output the resultant point cloud model dataset containing the estimated features
  */
void
  pcl::Filter<sensor_msgs::PointCloud2>::filter (PointCloud &output)
{
  if (!initCompute ()) 
    return;

  // If no point indices have been given, construct a set of indices for the entire input point cloud
  if (!indices_)
  {
//          // Resize the output dataset
//          if (output->points.size () != indices_->size ())
//            output->points.resize (indices_->size ());
  }

//        // Resize the output dataset
//        if (output->points.size () != indices_->size ())
//          output->points.resize (indices_->size ());
//        output.fields   = input_->fields;
//        output.header   = input_->header;
//        output.width    = input_->width;
//        output.height   = input_->height;
//        output.is_dense = input_->is_dense;

  // Apply the actual filter
  applyFilter (output);

  deinitCompute ();
}

