/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2010, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: test_mls.cpp 32623 2010-09-13 14:47:43Z marton $
 *
 */

/**

\author Zoltan Csaba Marton and Radu Bogdan Rusu

**///#include <gtest/gtest.h>

#include <boost/make_shared.hpp>
#include <ros/ros.h>
#include <pcl/point_types.h>
#include <pcl/io/pcd_io.h>
#include <pcl/kdtree/kdtree_ann.h>
//#include <pcl/features/feature.h>
//#include <pcl/features/normal_3d_omp.h>
//#include <pcl/features/normal_3d_tbb.h>
//#include <pcl/features/moment_invariants.h>
//#include <pcl/features/boundary.h>
//#include <pcl/features/principal_curvatures.h>
//#include <pcl/features/pfh.h>
//#include <pcl/features/fpfh.h>
//#include <pcl/features/fpfh_omp.h>
//#include <pcl/features/feature.h>
#include <pcl/surface/mls.h>

using namespace pcl;
using namespace pcl::io;
using namespace std;

typedef KdTree<PointXYZ>::Ptr KdTreePtr;

PointCloud<PointXYZ> cloud;
vector<int> indices;
KdTreePtr tree;

/* ---[ */
int
  main (int argc, char** argv)
{
  // Input
  sensor_msgs::PointCloud2 cloud_blob;
  loadPCDFile ("./test/bun0.pcd", cloud_blob);
  fromROSMsg (cloud_blob, cloud);

  // Indices
  indices.resize (cloud.points.size ());
  for (size_t i = 0; i < indices.size (); ++i) { indices[i] = i; }

  // KD-Tree
  tree = boost::make_shared<KdTreeANN<PointXYZ> > ();
  tree->setInputCloud (boost::make_shared<PointCloud<PointXYZ> > (cloud));

  // Init objects
  PointCloud<PointNormal> mls_points;
  MovingLeastSquares<PointXYZ, PointNormal> mls;

  // Set parameters
  mls.setInputCloud (boost::make_shared <const PointCloud<PointXYZ> > (cloud));
  mls.setIndices (boost::make_shared <vector<int> > (indices));
  mls.setPolynomialFit (true);
  mls.setSearchMethod (tree);
  mls.setSearchRadius (0.005);

  // Reconstruct
  mls.reconstruct (mls_points);

  // Save output
  savePCDFile ("./test/bun0-mls.pcd", mls_points);
  return (0);
}
/* ]--- */
