/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2009, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: voxel_grid.h 32874 2010-09-26 18:16:25Z rusu $
 *
 */

#ifndef PCL_ROS_FILTERS_VOXEL_H_
#define PCL_ROS_FILTERS_VOXEL_H_

// PCL includes
#include <pcl/filters/voxel_grid.h>
#include "pcl_ros/filters/filter.h"

// Dynamic reconfigure
#include "pcl_ros/VoxelGridConfig.h"

namespace pcl_ros
{
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /** \brief @b VoxelGrid assembles a local 3D grid over a given PointCloud, and downsamples + filters the data.
    * \author Radu Bogdan Rusu
    */
  template <typename PointT>
  class VoxelGrid: virtual public Filter<PointT>, virtual public pcl::VoxelGrid<PointT>
  {
    using Filter<PointT>::getName;

    typedef typename Filter<PointT>::PointCloud PointCloud;

    protected:
      // Members derived from the base class
      using pcl::VoxelGrid<PointT>::leaf_size_;
      using pcl::VoxelGrid<PointT>::getLeafSize;
      using pcl::VoxelGrid<PointT>::setLeafSize;
      using pcl::VoxelGrid<PointT>::setFilterLimits;
      using pcl::VoxelGrid<PointT>::getFilterLimits;
      using pcl::VoxelGrid<PointT>::getFilterFieldName;
      using pcl::VoxelGrid<PointT>::setFilterFieldName;

      /** \brief Pointer to a dynamic reconfigure service. */
      boost::shared_ptr <dynamic_reconfigure::Server<pcl_ros::VoxelGridConfig> > srv_;

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Child initialization routine.
        * \param nh ROS node handle
        */
      bool child_init (ros::NodeHandle &nh, bool &has_service);

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Dynamic reconfigure callback
        * \param config the config object
        * \param level the dynamic reconfigure level
        */
      void config_callback (pcl_ros::VoxelGridConfig &config, uint32_t level);
  };

  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /** \brief @b VoxelGrid assembles a local 3D grid over a given PointCloud, and downsamples + filters the data.
    * \author Radu Bogdan Rusu
    */
  template <>
  class VoxelGrid<sensor_msgs::PointCloud2> : virtual public Filter<sensor_msgs::PointCloud2>, virtual public pcl::VoxelGrid<sensor_msgs::PointCloud2>
  {
    using Filter<sensor_msgs::PointCloud2>::getName;

    typedef Filter<sensor_msgs::PointCloud2>::PointCloud PointCloud;

    protected:
      /** \brief Pointer to a dynamic reconfigure service. */
      boost::shared_ptr <dynamic_reconfigure::Server<pcl_ros::VoxelGridConfig> > srv_;

      bool child_init (ros::NodeHandle &nh, bool &has_service);

      void config_callback (pcl_ros::VoxelGridConfig &config, uint32_t level);
  };
}

#include "pcl_ros/filters/voxel_grid.hpp"

#endif  //#ifndef PCL_ROS_FILTERS_VOXEL_H_
