/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2010, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: convex_hull.h 32052 2010-08-27 02:19:30Z rusu $
 *
 */

#ifndef PCL_ROS_CONVEX_HULL_2D_H_
#define PCL_ROS_CONVEX_HULL_2D_H_

#include "pcl_ros/pcl_nodelet.h"

// PCL includes
#include <pcl/surface/convex_hull.h>

// Dynamic reconfigure
#include <dynamic_reconfigure/server.h>

namespace pcl_ros
{
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  /** \brief @b ConvexHull2D represents a 2D ConvexHull implementation.
    * \author Radu Bogdan Rusu
    */
  template <typename PointInT, typename PointOutT>
  class ConvexHull2D: virtual public pcl::ConvexHull2D<PointInT, PointOutT>, virtual public PCLNodelet<PointInT>
  {
    using PCLNodelet<PointInT>::getName;
    using PCLNodelet<PointInT>::isValid;
    using PCLNodelet<PointInT>::indices_;
    using PCLNodelet<PointInT>::use_indices_;
    using PCLNodelet<PointInT>::sub_input_;
    using PCLNodelet<PointInT>::pub_output_;
    using PCLNodelet<PointInT>::sync_input_indices_a_;
    using PCLNodelet<PointInT>::sync_input_indices_e_;
    using PCLNodelet<PointInT>::input_;
    using PCLNodelet<PointInT>::max_queue_size_;
    using PCLNodelet<PointInT>::sub_input_filter_;
    using PCLNodelet<PointInT>::sub_indices_filter_;
    using PCLNodelet<PointInT>::sync_input_indices_a_;
    using PCLNodelet<PointInT>::sync_input_indices_e_;
    using PCLNodelet<PointInT>::approximate_sync_;
    using PCLNodelet<PointInT>::getMTPrivateNodeHandle;

    typedef sensor_msgs::PointCloud2 PointCloud2;
    typedef pcl::ConvexHull2D<PointInT, PointOutT> BaseClass;
    typedef typename BaseClass::PointCloudIn PointCloudIn;
    typedef typename PointCloudIn::Ptr PointCloudInPtr;
    typedef typename PointCloudIn::ConstPtr PointCloudInConstPtr;
    typedef typename BaseClass::PointCloudOut PointCloudOut;

    protected:
      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Reconstruct the surface and publish the results. Internal method. */
      void reconstructAndPublish ();

    private:
      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Nodelet initialization routine. */
      virtual void onInit ();

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Input point cloud callback. Used when \a use_indices is set.
        * \param cloud the pointer to the input point cloud
        */
      void input_callback (const PointCloudInConstPtr &cloud);

      //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      /** \brief Input point cloud callback.
        * \param cloud the pointer to the input point cloud
        * \param indices the pointer to the input point cloud indices
        */
      void input_indices_callback (const PointCloud2::ConstPtr &cloud, const pcl::PointIndicesConstPtr &indices);
  };
}

#include "pcl_ros/surface/convex_hull.hpp"

#endif  //#ifndef PCL_ROS_CONVEX_HULL_2D_H_
