/*
 * Software License Agreement (BSD License)
 *
 *  Copyright (c) 2009, Willow Garage, Inc.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of Willow Garage, Inc. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNERff OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: feature.cpp 33240 2010-03-11 00:54:05Z rusu $
 *
 */

/**

\author Radu Bogdan Rusu

**/

#include <pluginlib/class_list_macros.h>
#include <pcl/point_types.h>
#include "pcl_ros/features/feature.h"
#include "pcl_ros/features/boundary.h"
#include "pcl_ros/features/moment_invariants.h"
#include "pcl_ros/features/normal_3d.h"
#include "pcl_ros/features/normal_3d_omp.h"
#include "pcl_ros/features/normal_3d_tbb.h"
#include "pcl_ros/features/pfh.h"
#include "pcl_ros/features/principal_curvatures.h"
#include "pcl_ros/features/fpfh.h"
#include "pcl_ros/features/fpfh_omp.h"
#include "pcl_ros/features/vfh.h"

// @todo Change all PointXYZ to PointXYZW/PointXYZRGBA
typedef pcl_ros::NormalEstimation<pcl::PointXYZ,pcl::Normal> NormalEstimation;
typedef pcl_ros::NormalEstimationOMP<pcl::PointXYZ,pcl::Normal> NormalEstimationOMP;
typedef pcl_ros::NormalEstimationTBB<pcl::PointXYZ,pcl::Normal> NormalEstimationTBB;
typedef pcl_ros::MomentInvariantsEstimation<pcl::PointXYZ,pcl::MomentInvariants> MomentInvariantsEstimation;
typedef pcl_ros::BoundaryEstimation<pcl::PointXYZ,pcl::Normal,pcl::Boundary> BoundaryEstimation;
typedef pcl_ros::PFHEstimation<pcl::PointXYZ,pcl::Normal,pcl::PFHSignature125> PFHEstimation;
typedef pcl_ros::PrincipalCurvaturesEstimation<pcl::PointXYZ,pcl::Normal,pcl::PrincipalCurvatures> PrincipalCurvaturesEstimation;
typedef pcl_ros::FPFHEstimation<pcl::PointXYZ,pcl::Normal,pcl::FPFHSignature33> FPFHEstimation;
typedef pcl_ros::FPFHEstimationOMP<pcl::PointXYZ,pcl::Normal,pcl::FPFHSignature33> FPFHEstimationOMP;
typedef pcl_ros::VFHEstimation<pcl::PointXYZ,pcl::Normal,pcl::VFHSignature308> VFHEstimation;

PLUGINLIB_DECLARE_CLASS (pcl, NormalEstimation, NormalEstimation, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, NormalEstimationOMP, NormalEstimationOMP, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, NormalEstimationTBB, NormalEstimationTBB, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, MomentInvariantsEstimation, MomentInvariantsEstimation, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, BoundaryEstimation, BoundaryEstimation, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, PFHEstimation, PFHEstimation, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, PrincipalCurvaturesEstimation, PrincipalCurvaturesEstimation, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, FPFHEstimation, FPFHEstimation, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, FPFHEstimationOMP, FPFHEstimationOMP, nodelet::Nodelet);
PLUGINLIB_DECLARE_CLASS (pcl, VFHEstimation, VFHEstimation, nodelet::Nodelet);

